#
# confs.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'racc/scanner'


module EST

class ConfigFileScanner < Racc::Scanner

  DEBUG = false


  COMMENT = /\A\#[^\r\n]*/
  WORD    = /\A[^\s":={}\[\]\#]+/
  QBEG    = /\A"/

  def initialize( str )
    super

    skip_all_spc
    if @scan.empty? then
      scan_error! "empty config file"
    end
  end


  def skip_all_spc
    while true do
      return if @scan.empty?
      if    @scan.skip SPACE   then
      elsif @scan.skip EOL     then @lineno += 1
      elsif @scan.skip COMMENT then
      else
        break
      end
    end
  end

  def scan
    tmp = ret = nil

    while true do

      if @scan.empty? then
        ret = [ false, '$end' ]
        break
      end

      if @scan.skip( /\A[ \t]+/ ) then
        ;

      elsif tmp = @scan.scan( WORD ) then
        ret = [ :STRING, tmp ]
        break
      
      elsif tmp = @scan.skip( QBEG ) then
        tmp = scan_string( '"', true )
        ret = [ :QUOTED, tmp ]
        break

      elsif tmp = @scan.skip( EOL ) then
        @lineno += 1
        skip_all_spc
        ret = [ :EOL, '$eol' ]
        break

      elsif @scan.skip( COMMENT ) then
        ;

      else
        tmp = @scan.getch
        ret = [ tmp, tmp ]
        break
      end
    end

    debug_report ret if DEBUG
    ret
  end

end

end
